package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.distributed.print.business.bean.dto.excel.UserImportDto;
import com.qmth.distributed.print.business.entity.BasicSchool;
import com.qmth.distributed.print.business.entity.SysUser;
import com.qmth.distributed.print.business.service.BasicSchoolService;
import com.qmth.distributed.print.business.service.CommonService;
import com.qmth.distributed.print.business.service.SysUserService;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.Base64Util;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    CommonService commonService;

    /**
     * 执行导入用户逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Gson gson = new Gson();
        LinkedMultiValueMap<Long, SysUser> tbUserLinkedMultiValueMap = new LinkedMultiValueMap<>();
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> userImportDtoList = excelMap.get(i);
            for (int y = 0; y < userImportDtoList.size(); y++) {
                UserImportDto userImportDto = (UserImportDto) userImportDtoList.get(y);
                SysUser sysUser = gson.fromJson(gson.toJson(userImportDto), SysUser.class);

                //查用户登录名是否重复
                QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
                sysUserQueryWrapper.lambda().eq(SysUser::getLoginName, userImportDto.getLoginName());
                long count = sysUserService.count(sysUserQueryWrapper);
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[登录名]重复")));
                }
                //查学校是否存在
                if (Objects.nonNull(userImportDto.getSchoolCode())) {
                    QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
                    basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, userImportDto.getSchoolCode());
                    BasicSchool basicSchool = basicSchoolService.getOne(basicSchoolQueryWrapper);
                    if (Objects.isNull(basicSchool)) {
                        throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[学校代码]不存在")));
                    }
                    sysUser.setSchoolId(basicSchool.getId());
                }
                sysUser.setInsertInfo(requestUser.getId());
                sysUser.setPassword(Base64Util.encode(userImportDto.getPassword().getBytes()));
                tbUserLinkedMultiValueMap.add(userImportDto.getSysRole().getId(), sysUser);
            }
        }
        tbUserLinkedMultiValueMap.forEach((k, v) -> {
            sysUserService.saveBatch(v);
            v.forEach(s -> {
                commonService.addUserRolePrivilege(s, k);
            });
        });
        return map;
    }
}
